# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#

import re

import requests

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import nick2fas, avatar_url

from fedmsg_meta_fedora_infrastructure.conglomerators.meetbot import \
        meetbot as meetbot_conglomerator

blacklisted_people = [
    'zodbot',
]

link_regex = re.compile('(https?:\/\/[a-z0-9\.:].*?)[ $]')

class SupybotProcessor(BaseProcessor):
    __name__ = "meetbot"
    __description__ = "the Fedora IRC bot"
    __link__ = "https://meetbot.fedoraproject.org/"
    __docs__ = "https://fedoraproject.org/wiki/Zodbot"
    __obj__ = "IRC Meetings"
    __icon__ = "https://apps.fedoraproject.org/img/icons/meetbot.png"

    conglomerators = [
        meetbot_conglomerator.ByURL,
    ]

    def link(self, msg, **config):
        if 'meetbot.meeting.complete' in msg['topic']:
            return msg['msg']['url'].replace('http://', 'https://') + ".html"
        elif 'meetbot.meeting.item.link' in msg['topic']:
            line = msg['msg']['details']['line']
            results = link_regex.findall(line)
            if not results:
                return None
            return results[0]
        else:
            return None

    def long_form(self, msg, **config):
        if '.meeting.complete' in msg['topic']:
            url = msg['msg']['url'] + '.txt'
            response = requests.get(url)
            if response.status_code == 200:
                return response.text

    def subtitle(self, msg, **config):
        action = None
        agent = None
        line = None

        if 'meetbot.meeting.start' in msg['topic']:
            if msg['msg']['meeting_topic']:
                tmpl = self._('{owner} started meeting "{name}" in {channel}')
            else:
                tmpl = self._('{owner} started a meeting in {channel}')

        elif 'meetbot.meeting.complete' in msg['topic']:
            if msg['msg']['meeting_topic']:
                tmpl = self._('{owner}\'s meeting titled "{name}" '
                              'ended in {channel}')
            else:
                tmpl = self._('{owner}\'s meeting ended in {channel}')

        elif 'meetbot.meeting.topic.update' in msg['topic']:
            if msg['msg']['meeting_topic']:
                tmpl = self._('The topic of {owner}\'s "{name}" meeting '
                              'changed to "{topic}" in {channel}')
            else:
                tmpl = self._('The topic of {owner}\'s meeting '
                              'changed to "{topic}" in {channel}')
        elif 'meetbot.meeting.item.' in msg['topic']:
            if msg['msg']['meeting_topic']:
                tmpl = self._('{agent} {action} in the "{name}" meeting '
                              'in {channel}: "{line}"')
            else:
                tmpl = self._('{agent} {action} in a meeting '
                              'in {channel}: "{line}"')
            key = msg['topic'].rsplit('.')[-1]
            action_lookup = {
                'agreed': 'noted agreement',
                'accepted': 'accepted an item',
                'rejected': 'rejected an item',
                'action': 'noted an action',
                'info': 'raised a point of information',
                'idea': 'proposed an idea',
                'help': 'called for help',
                'link': 'linked to more information',
            }
            action = action_lookup[key]
            line = msg['msg']['details']['line']
        else:
            raise NotImplementedError("%r" % msg)

        if 'details' in msg['msg']:
            agent = nick2fas(msg['msg']['details']['nick'], **config)

        owner = nick2fas(msg['msg']['owner'], **config)
        name = msg['msg']['meeting_topic']
        channel = msg['msg']['channel']
        topic = msg['msg'].get('topic', 'no topic')

        return tmpl.format(owner=owner, agent=agent, name=name,
                           channel=channel, topic=topic,
                           action=action, line=line)

    def usernames(self, msg, **config):
        return set([
            nick2fas(nick, **config)
            for nick in msg['msg']['attendees']
            if nick not in blacklisted_people
        ])

    def secondary_icon(self, msg, **config):
        if 'details' in msg['msg']:
            user = nick2fas(msg['msg']['details']['nick'], **config)
        else:
            user = nick2fas(msg['msg']['owner'], **config)
        return avatar_url(user)

    def objects(self, msg, **config):
        objs = set([
            'attendees/' + nick2fas(person, **config)
            for person in msg['msg']['attendees']
            if person not in blacklisted_people
        ] + [
            'channels/' + msg['msg']['channel']
        ])

        if msg['msg']['meeting_topic']:
            objs.add('titles/' + msg['msg']['meeting_topic'])

        if msg['msg'].get('topic', None):
            objs.add('topics/' + msg['msg']['topic'])

        return objs



# This file is part of fedmsg.
# Copyright (C) 2017 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for MBS messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestModuleBuildStateChange(Base):
    """ These messages are published by the `module build service
    <https://fedoraproject.org/wiki/Changes/ModuleBuildService>`_ when a module
    build transitions between states (like starting, or completing).
    """
    expected_title = "mbs.module.state.change"
    expected_subti = "ralph's build of modules/testmodule " + \
        "entered the wait state."
    expected_link = "https://mbs.fedoraproject.org/module-build-service/1/module-builds/2"
    expected_icon = "https://apps.fedoraproject.org/img/icons/modularity.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([
        # TODO -- we'll eventually have to think about how to show all relevant
        # packages here.  Are "modules" really a package, from this PoV?
    ])
    expected_usernames = set(['ralph'])
    expected_objects = set(['testmodule'])
    msg = {
        "source_name": "datanommer",
        "i": 1,
        "timestamp": 1487188431.0,
        "msg_id": "2017-859a8cfb-6d95-43f4-9c0b-897e833b06b5",
        "topic": "org.fedoraproject.stg.mbs.module.state.change",
        "source_version": "0.6.5",
        "msg": {
            "state_reason": None,
            "component_builds": [
                19,
                20,
                21
            ],
            "name": "testmodule",
            "stream": "master",
            "time_submitted": 1487178787.0,
            "state_url": "/module-build-service/1/module-builds/2",
            "time_modified": 1487188430.0,
            "scmurl": "git://pkgs.stg.fedoraproject.org/modules/testmodule.git?#620ec77",
            "state": 1,
            "modulemd": "full modulemd text goes here...",
            "time_completed": None,
            "version": "20161115001755",
            "owner": "ralph",
            "id": 2,
            "state_name": "wait"
        }
    }



add_doc(locals())

if __name__ == '__main__':
    unittest.main()



import fedmsg.tests.test_meta

import arrow


class TestMailmanConglomerateByMessageId(
        fedmsg.tests.test_meta.ConglomerateBase):
    expected = [
        {
            'categories': set(['mailman']),
            'end_time': 1450110137.0,
            'human_time': arrow.get(1450058909.6923077).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/4OTZIFLXO7NQFH47S6BLPN45SBHOEA6P/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/f73b95bc363f96a24ec56c1cca611dff?s=64&d=retro',
            'start_time': 1449998105.0,
            'subjective': u'ctubbsii-fedora@apache.org, fdeutsch@redhat.com, and 6 others wrote 13 replies to the "Easier %config management?" thread on the devel list',
            'subtitle': u'ctubbsii-fedora@apache.org, fdeutsch@redhat.com, and 6 others wrote 13 replies to the "Easier %config management?" thread on the devel list',
            'timestamp': 1450058909.6923077,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set([])
        }, {
            'categories': set(['mailman']),
            'end_time': 1450107961.0,
            'human_time': arrow.get(1450107961.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/N5VA2ZFGOQBKWUWWEIL4TBFWIT7Q74E2/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/6382b3cc881412b77bfcaeed026001c00d9e3025e66c20f6e7e92f079851462a?s=64&d=retro',
            'start_time': 1450107961.0,
            'subjective': u"nobody wrote '[POC-change] Fedora packages point of contact updates' to the devel list",
            'subtitle': u"nobody wrote '[POC-change] Fedora packages point of contact updates' to the devel list",
            'timestamp': 1450107961.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set(['nobody']),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450106384.0,
            'human_time': arrow.get(1450106384.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/MWGTN2OLUOKCFYDS4XNFMWHUJ3C5FHT5/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/71630da8f590ff926d611abe228bd7f4a8935f09908ebe69ba3a2362c2d6fc5c?s=64&d=retro',
            'start_time': 1450106384.0,
            'subjective': u"rawhide wrote 'Fedora Rawhide 20151214 compose check report' to the devel list",
            'subtitle': u"rawhide wrote 'Fedora Rawhide 20151214 compose check report' to the devel list",
            'timestamp': 1450106384.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set(['rawhide']),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450104230.0,
            'human_time': arrow.get(1450104230.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/GLDIH254AMANCVIQVOJ2JNOVAU6C32IT/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/579f49f7c4d9f8763d6733e31568dd1a?s=64&d=retro',
            'start_time': 1450104230.0,
            'subjective': u"On the devel list, pwouters@redhat.com replied to 'Re: F24 System Wide Change: Default Local DNS Resolver'",
            'subtitle': u"On the devel list, pwouters@redhat.com replied to 'Re: F24 System Wide Change: Default Local DNS Resolver'",
            'timestamp': 1450104230.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set([]),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450093963.0,
            'human_time': arrow.get(1450093963.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/BLDNP6AOSH2GH3V5OUWFOLMNLMC46OVT/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/eb3db1dface818fe81cc1dce5e8ac680?s=64&d=retro',
            'start_time': 1450093963.0,
            'subjective': u"jkurik@redhat.com wrote '[Elections] Reminder : Voting Period Ends Today' to the devel list",
            'subtitle': u"jkurik@redhat.com wrote '[Elections] Reminder : Voting Period Ends Today' to the devel list",
            'timestamp': 1450093963.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set([]),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450089571.0,
            'human_time': arrow.get(1450089571.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/YLFK7I4HTRJ6BBXOCMG5ZX5HCT3SNBSN/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/71630da8f590ff926d611abe228bd7f4a8935f09908ebe69ba3a2362c2d6fc5c?s=64&d=retro',
            'start_time': 1450089571.0,
            'subjective': u"rawhide wrote 'rawhide report: 20151214 changes' to the devel list",
            'subtitle': u"rawhide wrote 'rawhide report: 20151214 changes' to the devel list",
            'timestamp': 1450089571.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set(['rawhide']),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450062083.0,
            'human_time': arrow.get(1450062083.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/UQF6BKC5E22TDEL34TIXGTCEPSTVXSCM/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/8213db2eb8fc1e9d78f70bcba52e4b379e44f662792e0e0407fb67ff168aeaee?s=64&d=retro',
            'start_time': 1450062083.0,
            'subjective': u"adamwill wrote '[Test-Announce] Proposal to CANCEL: 2015-12-14 Fedora QA Meeting' to the devel list",
            'subtitle': u"adamwill wrote '[Test-Announce] Proposal to CANCEL: 2015-12-14 Fedora QA Meeting' to the devel list",
            'timestamp': 1450062083.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set(['adamwill']),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450019998.0,
            'human_time': arrow.get(1450019998.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/WXYKNWQN5HOMEP2XDQOSH56RK4JHBJUO/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/71630da8f590ff926d611abe228bd7f4a8935f09908ebe69ba3a2362c2d6fc5c?s=64&d=retro',
            'start_time': 1450019998.0,
            'subjective': u"rawhide wrote 'Fedora Rawhide 20151213 compose check report' to the devel list",
            'subtitle': u"rawhide wrote 'Fedora Rawhide 20151213 compose check report' to the devel list",
            'timestamp': 1450019998.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set(['rawhide']),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450015355.0,
            'human_time': arrow.get(1450015355.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/HPBHTMMCDAUU5ZMTUJCPFIERWZEHDNGG/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/8de2bbf335b88faa16fefdb63281ed0c?s=64&d=retro',
            'start_time': 1450015355.0,
            'subjective': u"On the devel list, jpesco@gmx.com replied to 'Re: REMINDER: FESCo, Council, FAmSCo elections'",
            'subtitle': u"On the devel list, jpesco@gmx.com replied to 'Re: REMINDER: FESCo, Council, FAmSCo elections'",
            'timestamp': 1450015355.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set([]),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450012283.0,
            'human_time': arrow.get(1450012283.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/3UGB7MTFX6KBXHUNYNRG3EJ5JB5T2HNP/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/72d9d1481691ed08bf26954b3485d27d?s=64&d=retro',
            'start_time': 1450012283.0,
            'subjective': u"On the devel list, mschwendt@gmail.com replied to 'Re: dnf behavior expected?'",
            'subtitle': u"On the devel list, mschwendt@gmail.com replied to 'Re: dnf behavior expected?'",
            'timestamp': 1450012283.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set([]),
        }, {
            'categories': set(['mailman']),
            'end_time': 1450002803.0,
            'human_time': arrow.get(1450002803.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/RDVKKW5QZZBFUXVQR32ZQ7N3DXMTBRRU/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/71630da8f590ff926d611abe228bd7f4a8935f09908ebe69ba3a2362c2d6fc5c?s=64&d=retro',
            'start_time': 1450002803.0,
            'subjective': u"rawhide wrote 'rawhide report: 20151213 changes' to the devel list",
            'subtitle': u"rawhide wrote 'rawhide report: 20151213 changes' to the devel list",
            'timestamp': 1450002803.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set(['rawhide']),
        }, {
            'categories': set(['mailman']),
            'end_time': 1449994343.0,
            'human_time': arrow.get(1449994281.0).humanize(),
            'icon': 'https://apps.fedoraproject.org/img/icons/hyperkitty.png',
            'link': 'http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/G6TQAZYKZF3KH6XW3SV6ZSW3MPARYG54/',
            'packages': set([]),
            'secondary_icon': 'https://seccdn.libravatar.org/avatar/5640a90bc0024f3300736f1a3527c9d1?s=64&d=retro',
            'start_time': 1449994219.0,
            'subjective': u'Bjorn@xn--rombobjrn-67a.se wrote 2 replies to the "Can Koji handle a soname change and a self-dependency?" thread on the devel list',
            'subtitle': u'Bjorn@xn--rombobjrn-67a.se wrote 2 replies to the "Can Koji handle a soname change and a self-dependency?" thread on the devel list',
            'timestamp': 1449994281.0,
            'topics': set(['org.fedoraproject.prod.mailman.receive']),
            'usernames': set([]),
        }
    ]


    originals = [
        {
            "i": 1285,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/OFFAFTZZ6WU6SAAHDTYQRYIH76QFWPVN/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Reindl Harald <h.reindl@thelounge.net>",
                    "in-reply-to": "<CAL5zq9bt8_aF-AhJHZsdAj-4gYs3cprq6XiCkG5bE1mqroFZdg@mail.gmail.com>",
                    "message-id": "<566EECA4.1060109@thelounge.net>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>\n <566DD742.50607@thelounge.net>\n <CAL5zq9bt8_aF-AhJHZsdAj-4gYs3cprq6XiCkG5bE1mqroFZdg@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "OFFAFTZZ6WU6SAAHDTYQRYIH76QFWPVN"
                }
            },
            "msg_id": "2015-d5fb3bff-a793-4cde-8ced-e63309bbba76",
            "timestamp": 1450110137.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1269,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/QIVQKUUJ5GJVFHOL7H7QSHLSV75MSBXF/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Christopher <ctubbsii-fedora@apache.org>",
                    "in-reply-to": "<566DD742.50607@thelounge.net>",
                    "message-id": "<CAL5zq9bt8_aF-AhJHZsdAj-4gYs3cprq6XiCkG5bE1mqroFZdg@mail.gmail.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com> <566DD742.50607@thelounge.net>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "QIVQKUUJ5GJVFHOL7H7QSHLSV75MSBXF"
                }
            },
            "msg_id": "2015-a4d4fabb-3ee4-4451-9f72-a668794c49a7",
            "timestamp": 1450108932.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1264,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/N5VA2ZFGOQBKWUWWEIL4TBFWIT7Q74E2/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "nobody@fedoraproject.org",
                    "in-reply-to": None,
                    "message-id": "<20151214100003.E03BD6099882@sundries01.phx2.fedoraproject.org>",
                    "references": None,
                    "subject": "[POC-change] Fedora packages point of contact updates",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "nonmember-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop; member-moderation; header-match-40538; header-match-40539; header-match-40540; header-match-40541; header-match-40542; header-match-40543; header-match-40544",
                    "x-message-id-hash": "N5VA2ZFGOQBKWUWWEIL4TBFWIT7Q74E2"
                }
            },
            "msg_id": "2015-6e19ea40-2e0a-44f1-a2eb-0bce74de2070",
            "timestamp": 1450107961.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1253,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/FWGMQLVTDSXRT5RCI4YHIJ5TLC2EMPQH/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Christopher <ctubbsii-fedora@apache.org>",
                    "in-reply-to": "<566DCBFD.3000704@thelounge.net>",
                    "message-id": "<CAL5zq9ZK4hdBqXuzrUcaowLfnSirHxg8+Oaf4yxTrMUbqJcRwQ@mail.gmail.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <566DCBFD.3000704@thelounge.net>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "FWGMQLVTDSXRT5RCI4YHIJ5TLC2EMPQH"
                }
            },
            "msg_id": "2015-191c3c40-3e9f-47e9-b40d-c820b92d77d2",
            "timestamp": 1450106460.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1251,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/MWGTN2OLUOKCFYDS4XNFMWHUJ3C5FHT5/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Fedora compose checker <rawhide@fedoraproject.org>",
                    "in-reply-to": None,
                    "message-id": "<20151214151926.4B8446062BF8@bastion01.phx2.fedoraproject.org>",
                    "references": None,
                    "subject": "Fedora Rawhide 20151214 compose check report",
                    "to": "test@lists.fedoraproject.org, devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "MWGTN2OLUOKCFYDS4XNFMWHUJ3C5FHT5"
                }
            },
            "msg_id": "2015-a9720f64-76e8-4274-8d9a-f87ee78ae25a",
            "timestamp": 1450106384.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1213,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/GLDIH254AMANCVIQVOJ2JNOVAU6C32IT/>",
                    "cc": "devel@lists.fedoraproject.org",
                    "delivered-to": None,
                    "from": "Paul Wouters <pwouters@redhat.com>",
                    "in-reply-to": "<6907000.gmhhBm2oPe@neon.home.il>",
                    "message-id": "<566ED390.8040603@redhat.com>",
                    "references": "<CA+Sa5FoQW2OYxWoTwWj3qrJ+rVEiievEqF3qP0G+5H3VA4JtMQ@mail.gmail.com>\n <22305478.k9qG4LVH1Z@neon.home.il> <566AD918.8070705@redhat.com>\n <6907000.gmhhBm2oPe@neon.home.il>",
                    "subject": "Re: F24 System Wide Change: Default Local DNS Resolver",
                    "to": "Oron Peled <oron@actcom.co.il>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "GLDIH254AMANCVIQVOJ2JNOVAU6C32IT"
                }
            },
            "msg_id": "2015-287651b5-ed46-4943-9241-59ceb9aca191",
            "timestamp": 1450104230.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1082,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/BLDNP6AOSH2GH3V5OUWFOLMNLMC46OVT/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Jan Kurik <jkurik@redhat.com>",
                    "in-reply-to": None,
                    "message-id": "<CA+Sa5FpvRehsPg_0jcKP79w5BDApxNSbM-yNFa3uWSM_ORE3AA@mail.gmail.com>",
                    "references": None,
                    "subject": "[Elections] Reminder : Voting Period Ends Today",
                    "to": "devel-announce@lists.fedoraproject.org, \n\tDevelopment discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "BLDNP6AOSH2GH3V5OUWFOLMNLMC46OVT"
                }
            },
            "msg_id": "2015-26597411-4eca-4c4a-81e0-573a9308650d",
            "timestamp": 1450093963.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1056,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/YLFK7I4HTRJ6BBXOCMG5ZX5HCT3SNBSN/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Fedora Rawhide Report <rawhide@fedoraproject.org>",
                    "in-reply-to": None,
                    "message-id": "<20151214103852.GA26839@rawhide-composer.phx2.fedoraproject.org>",
                    "references": None,
                    "subject": "rawhide report: 20151214 changes",
                    "to": "devel@lists.fedoraproject.org, test@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "YLFK7I4HTRJ6BBXOCMG5ZX5HCT3SNBSN"
                }
            },
            "msg_id": "2015-49e3b336-a560-46d4-8a2b-5e705073dd16",
            "timestamp": 1450089571.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 1038,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/4OTZIFLXO7NQFH47S6BLPN45SBHOEA6P/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Fabian Deutsch <fdeutsch@redhat.com>",
                    "in-reply-to": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "message-id": "<CA+PVUaRf=1fM7djiP8Vtm97OjnyHBLAxpwDRvw6X5mwGOY+Kbg@mail.gmail.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "4OTZIFLXO7NQFH47S6BLPN45SBHOEA6P"
                }
            },
            "msg_id": "2015-c9d1afb8-0a80-4d6d-b389-76962b403f5f",
            "timestamp": 1450086838.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 984,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/XL7UKU733VSUMO3CDQDME2IXSIHCIY2G/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Panu Matilainen <pmatilai@laiskiainen.org>",
                    "in-reply-to": "<CAObL_7EYXGQP2Ow-jHj2gx=TBc1XOVSAw2=xoVk5q4nmB7o-Jg@mail.gmail.com>",
                    "message-id": "<566E7BAB.1010502@laiskiainen.org>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>\n <566DD742.50607@thelounge.net>\n <CAObL_7EYXGQP2Ow-jHj2gx=TBc1XOVSAw2=xoVk5q4nmB7o-Jg@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "XL7UKU733VSUMO3CDQDME2IXSIHCIY2G"
                }
            },
            "msg_id": "2015-fa768e7c-dc5f-4103-b59a-ec40dd634575",
            "timestamp": 1450081218.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 938,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/UQF6BKC5E22TDEL34TIXGTCEPSTVXSCM/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Adam Williamson <adamwill@fedoraproject.org>",
                    "in-reply-to": None,
                    "message-id": "<1450062061.7151.23.camel@fedoraproject.org>",
                    "references": None,
                    "subject": "[Test-Announce] Proposal to CANCEL: 2015-12-14 Fedora QA Meeting",
                    "to": "test-announce@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "UQF6BKC5E22TDEL34TIXGTCEPSTVXSCM"
                }
            },
            "msg_id": "2015-3b38a2b7-26c4-4f1b-979f-41807783bbbd",
            "timestamp": 1450062083.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 918,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/BPOKLZCOCUFWMHTWKREVXKXD7D3C5A7L/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Colin Walters <walters@verbum.org>",
                    "in-reply-to": "<20151213194131.GN26159@redhat.com>",
                    "message-id": "<1450044781.1619079.466286985.71F765CA@webmail.messagingengine.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <20151213194131.GN26159@redhat.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "BPOKLZCOCUFWMHTWKREVXKXD7D3C5A7L"
                }
            },
            "msg_id": "2015-75b64fc8-eb63-41ca-b9f3-87c8704a7e2e",
            "timestamp": 1450044801.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 914,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/GZIKG6EFAPWEAKBPEMQ5KVYYLJG6WTEI/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Reindl Harald <h.reindl@thelounge.net>",
                    "in-reply-to": "<CAObL_7EYXGQP2Ow-jHj2gx=TBc1XOVSAw2=xoVk5q4nmB7o-Jg@mail.gmail.com>",
                    "message-id": "<566DDB7E.1080505@thelounge.net>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>\n <566DD742.50607@thelounge.net>\n <CAObL_7EYXGQP2Ow-jHj2gx=TBc1XOVSAw2=xoVk5q4nmB7o-Jg@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "GZIKG6EFAPWEAKBPEMQ5KVYYLJG6WTEI"
                }
            },
            "msg_id": "2015-cbe5cf35-e092-40e4-ae24-f953a4579f7f",
            "timestamp": 1450040210.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 913,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/M34UJ72IBEYB53BZVYNCDQJMEWOEYV6J/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Andrew Lutomirski <luto@mit.edu>",
                    "in-reply-to": "<566DD742.50607@thelounge.net>",
                    "message-id": "<CAObL_7EYXGQP2Ow-jHj2gx=TBc1XOVSAw2=xoVk5q4nmB7o-Jg@mail.gmail.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n\t<CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>\n\t<566DD742.50607@thelounge.net>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "M34UJ72IBEYB53BZVYNCDQJMEWOEYV6J"
                }
            },
            "msg_id": "2015-2b4600fa-1813-407a-8283-f539ba25808c",
            "timestamp": 1450039852.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 912,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/NNJBSF7M4AYB67FUUIO4LWV3MCJ7ANMX/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Reindl Harald <h.reindl@thelounge.net>",
                    "in-reply-to": "<CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>",
                    "message-id": "<566DD742.50607@thelounge.net>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "NNJBSF7M4AYB67FUUIO4LWV3MCJ7ANMX"
                }
            },
            "msg_id": "2015-a088bcf9-b796-47eb-9418-5ac7e5055a0f",
            "timestamp": 1450039132.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 910,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/QBF6PDFCCH66ZCU4G4NR55QMQGWQIT24/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Andrew Lutomirski <luto@mit.edu>",
                    "in-reply-to": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "message-id": "<CAObL_7HgBZgh1W0yJb0Pmfa2pBcwoz+J4e9z5y_qKrSbLLEkMA@mail.gmail.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "QBF6PDFCCH66ZCU4G4NR55QMQGWQIT24"
                }
            },
            "msg_id": "2015-9cc72c62-90dd-4ee0-a876-5168e35b2065",
            "timestamp": 1450038189.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 906,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/6Q7Y35X5GUKGNS3YECNVYQDYR7OCBEL6/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Reindl Harald <h.reindl@thelounge.net>",
                    "in-reply-to": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "message-id": "<566DCBFD.3000704@thelounge.net>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "6Q7Y35X5GUKGNS3YECNVYQDYR7OCBEL6"
                }
            },
            "msg_id": "2015-64402233-3471-4de8-b909-db32c96de695",
            "timestamp": 1450036236.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 905,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/TKJB5SDTGMXVRV2DJ7XWK7ZKGX3HXMFN/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "\"Richard W.M. Jones\" <rjones@redhat.com>",
                    "in-reply-to": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "message-id": "<20151213194131.GN26159@redhat.com>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "TKJB5SDTGMXVRV2DJ7XWK7ZKGX3HXMFN"
                }
            },
            "msg_id": "2015-e73d5e41-46de-4f50-a4ce-7cc0b608ad53",
            "timestamp": 1450035716.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 873,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/WXYKNWQN5HOMEP2XDQOSH56RK4JHBJUO/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Fedora compose checker <rawhide@fedoraproject.org>",
                    "in-reply-to": None,
                    "message-id": "<20151213151942.93CC06076F73@bastion01.phx2.fedoraproject.org>",
                    "references": None,
                    "subject": "Fedora Rawhide 20151213 compose check report",
                    "to": "test@lists.fedoraproject.org, devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "WXYKNWQN5HOMEP2XDQOSH56RK4JHBJUO"
                }
            },
            "msg_id": "2015-ff061d3c-42c8-4a4e-9083-da16e7ffef8e",
            "timestamp": 1450019998.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 868,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/HPBHTMMCDAUU5ZMTUJCPFIERWZEHDNGG/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Joseph Pesco <jpesco@gmx.com>",
                    "in-reply-to": "<CAJCQCtRXk_-Sk0+825daQFWkjt39GBjb9moDFDdWTQyz7TRKOg@mail.gmail.com>",
                    "message-id": "<1450015318.3754.0.camel@gmx.com>",
                    "references": "\n\t<CA+Sa5FoBH8yewYNW=HKSG_NFmgwLs+uxDQ5_x=uX7E7aW4X9bQ@mail.gmail.com>\n\t <CAJCQCtT=rspdip--31GMFAQLeaKeZKiG6eZzWJuKybreur2+2w@mail.gmail.com>\n\t <1449858511.3952.51.camel@redhat.com>\n\t <CAJCQCtRXk_-Sk0+825daQFWkjt39GBjb9moDFDdWTQyz7TRKOg@mail.gmail.com>",
                    "subject": "Re: REMINDER: FESCo, Council, FAmSCo elections",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "HPBHTMMCDAUU5ZMTUJCPFIERWZEHDNGG"
                }
            },
            "msg_id": "2015-248bdd54-0f56-41bd-a226-8802479ae0c6",
            "timestamp": 1450015355.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 857,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/3UGB7MTFX6KBXHUNYNRG3EJ5JB5T2HNP/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Michael Schwendt <mschwendt@gmail.com>",
                    "in-reply-to": "<CAKHp9yqjoeijtK25Y8ROWQrMZqB2_nmLBv=psKCyJVR35bCmCA@mail.gmail.com>",
                    "message-id": "<20151213141102.40f3db30@noname>",
                    "references": "<CAKHp9ypJdqp_sc5S0KLcAE=QWbCeDdY5c24=ueyH0mHF+ouBRA@mail.gmail.com>\n\t<20151208165729.3d396e49@noname>\n\t<CAKHp9yqjoeijtK25Y8ROWQrMZqB2_nmLBv=psKCyJVR35bCmCA@mail.gmail.com>",
                    "subject": "Re: dnf behavior expected?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "3UGB7MTFX6KBXHUNYNRG3EJ5JB5T2HNP"
                }
            },
            "msg_id": "2015-aa785443-2e4c-4beb-b837-4c26df63f24c",
            "timestamp": 1450012283.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 832,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/RDVKKW5QZZBFUXVQR32ZQ7N3DXMTBRRU/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "Fedora Rawhide Report <rawhide@fedoraproject.org>",
                    "in-reply-to": None,
                    "message-id": "<20151213103242.GA4337@rawhide-composer.phx2.fedoraproject.org>",
                    "references": None,
                    "subject": "rawhide report: 20151213 changes",
                    "to": "devel@lists.fedoraproject.org, test@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "RDVKKW5QZZBFUXVQR32ZQ7N3DXMTBRRU"
                }
            },
            "msg_id": "2015-3796f3ae-7678-4963-a42d-101f861f5983",
            "timestamp": 1450002803.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 820,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/BHDNIRPDYET3WYNTMITN2BA2NLISYHJA/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "\"J. Randall Owens\" <jrowens.fedora@ghiapet.net>",
                    "in-reply-to": "<20151213080704.GA31870@host1.jankratochvil.net>",
                    "message-id": "<566D36FF.1020607@ghiapet.net>",
                    "references": "<CAL5zq9ZYgmi+BejiUSA1HKa+wj_CSfQfFFoGA-VZHnMJ3bPz_g@mail.gmail.com>\n <20151213080704.GA31870@host1.jankratochvil.net>",
                    "subject": "Re: Easier %config management?",
                    "to": "Development discussions related to Fedora <devel@lists.fedoraproject.org>",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "BHDNIRPDYET3WYNTMITN2BA2NLISYHJA"
                }
            },
            "msg_id": "2015-fe5fddb5-85df-4dd9-955d-5de1a68aea54",
            "timestamp": 1449998105.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 811,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/S7KLJGO6QO233SNPOOBV6MOBUCD7GRGZ/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "=?UTF-8?B?QmrDtnJu?= Persson <Bjorn@xn--rombobjrn-67a.se>",
                    "in-reply-to": "<20151207113244.7934fed7@speedy.xn--rombobjrn-67a.se>",
                    "message-id": "<20151213091203.6d8cc34c@hactar.xn--rombobjrn-67a.se>",
                    "references": "<20151129003537.10219c62@hactar.xn--rombobjrn-67a.se>\n\t<20151129144048.7943824c@sheelba.scrye.com>\n\t<20151201154626.10e417cf@speedy.xn--rombobjrn-67a.se>\n\t<20151204100457.4ecbb61f@sheelba.scrye.com>\n\t<20151207113244.7934fed7@speedy.xn--rombobjrn-67a.se>",
                    "subject": "Re: Can Koji handle a soname change and a self-dependency?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "S7KLJGO6QO233SNPOOBV6MOBUCD7GRGZ"
                }
            },
            "msg_id": "2015-1da1b4e6-e784-4f1f-937c-8846399ea534",
            "timestamp": 1449994343.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        },
        {
            "i": 810,
            "msg": {
                "mlist": {
                    "list_name": "devel"
                },
                "msg": {
                    "archived-at": "<http://lists.fedoraproject.org/archives/list/devel@lists.fedoraproject.org/message/G6TQAZYKZF3KH6XW3SV6ZSW3MPARYG54/>",
                    "cc": None,
                    "delivered-to": None,
                    "from": "=?UTF-8?B?QmrDtnJu?= Persson <Bjorn@xn--rombobjrn-67a.se>",
                    "in-reply-to": "<20151210232042.e00d69b0c02cd9986938caa6@danny.cz>",
                    "message-id": "<20151213090940.2d92c807@hactar.xn--rombobjrn-67a.se>",
                    "references": "<20151129003537.10219c62@hactar.xn--rombobjrn-67a.se>\n\t<20151129144048.7943824c@sheelba.scrye.com>\n\t<20151201154626.10e417cf@speedy.xn--rombobjrn-67a.se>\n\t<20151204100457.4ecbb61f@sheelba.scrye.com>\n\t<20151207113244.7934fed7@speedy.xn--rombobjrn-67a.se>\n\t<20151210232042.e00d69b0c02cd9986938caa6@danny.cz>",
                    "subject": "Re: Can Koji handle a soname change and a self-dependency?",
                    "to": "devel@lists.fedoraproject.org",
                    "x-mailman-rule-hits": "member-moderation",
                    "x-mailman-rule-misses": "approved; emergency; loop",
                    "x-message-id-hash": "G6TQAZYKZF3KH6XW3SV6ZSW3MPARYG54"
                }
            },
            "msg_id": "2015-273e2273-cdbe-4448-83fb-e133ead87f61",
            "timestamp": 1449994219.0,
            "topic": "org.fedoraproject.prod.mailman.receive"
        }
    ]



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class KernelTestProcessor(BaseProcessor):
    __name__ = "kerneltest"
    __description__ = "Kernel Testing Initiative"
    __link__ = "https://apps.fedoraproject.org/kerneltest"
    __icon__ = "https://apps.fedoraproject.org/img/icons/tux.png"
    __docs__ = "https://fedoraproject.org/wiki/KernelTestingInitiative"
    __obj__ = "Kernel Tests"

    def link(self, msg, **config):
        if 'test' in msg['msg']:
            vers = msg['msg']['test']['kernel_version']
            return 'https://apps.fedoraproject.org/kerneltest/kernel/' + vers
        return 'https://apps.fedoraproject.org/kerneltest/stats'

    def subtitle(self, msg, **config):
        user = msg['msg']['agent']

        if 'test' in msg['msg']:
            tmpl = self._('{user} ran a test of {version} ({result})')
            version = msg['msg']['test']['kernel_version']
            result = msg['msg']['test']['result']
            return tmpl.format(user=user, version=version, result=result)
        elif 'release.edit' in msg['topic']:
            tmpl = self._(
                '{user} edited the {release}-{support} '
                'release for kerneltest')
            release = msg['msg']['release']['releasenum']
            support = msg['msg']['release']['support']
            return tmpl.format(user=user, release=release, support=support)
        elif 'release.new' in msg['topic']:
            tmpl = self._(
                '{user} added a new release for kerneltest: '
                '{release} {support}')
            release = msg['msg']['release']['releasenum']
            support = msg['msg']['release']['support']
            return tmpl.format(user=user, release=release, support=support)

    def secondary_icon(self, msg, **config):
        return avatar_url(msg['msg']['agent'])

    def usernames(self, msg, **config):
        return set([msg['msg']['agent']])

    def packages(self, msg, **config):
        return set(['kernel'])

    def objects(self, msg, **config):
        if 'test' in msg['msg']:
            vers = msg['msg']['test']['kernel_version']
            return set(['kernel/' + vers])
        elif 'release' in msg['msg']:
            releasenum = msg['msg']['release']['releasenum']
            support = msg['msg']['release']['support']
            return set(['release/{releasenum}/{support}'.format(
                releasenum=releasenum, support=support)])



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class AskbotProcessor(BaseProcessor):
    __name__ = "Askbot"
    __description__ = "the Fedora 'Ask' Forum"
    __link__ = "https://ask.fedoraproject.org"
    __docs__ = "https://askbot.com"
    __icon__ = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    __obj__ = "Ask Fedora Qs & As"

    # No icon that I can find..
    #__icon__ = "https://apps.fedoraproject.org/packages/" + \
    #    "images/icons/package_128x128.png"

    def subtitle(self, msg, **config):
        user = msg['msg']['agent']
        title = ""
        if 'askbot.post.edit' in msg['topic']:
            title = msg['msg']['thread']['title']
            if msg['msg']['created']:
                if msg['msg']['post']['post_type'] == 'question':
                    tmpl = self._("{user} asked the question '{title}'")
                else:
                    tmpl = self._(
                        "{user} suggested an answer to the question '{title}'")
            else:
                if msg['msg']['post']['post_type'] == 'question':
                    tmpl = self._("{user} updated the question '{title}'")
                else:
                    tmpl = self._(
                        "{user} updated an answer to the question '{title}'")
        elif 'askbot.tag.update' in msg['topic']:
            title = msg['msg']['thread']['title']
            tmpl = self._("{user} altered tags on askbot question '{title}'")
        elif 'askbot.post.flag_offensive.add' in msg['topic']:
            title = msg['msg']['thread']['title']
            if msg['msg']['instance']['post_type'] == 'question':
                tmpl = self._("{user} flagged a question as offensive!")
            else:
                tmpl = self._("{user} flagged an answer as offensive!")
        elif 'askbot.post.flag_offensive.delete' in msg['topic']:
            if msg['msg']['instance']['post_type'] == 'question':
                tmpl = self._("{user} unflagged a question as offensive...")
            else:
                tmpl = self._("{user} unflagged an answer as offensive...")
        elif 'askbot.post.delete' in msg['topic']:
            title = msg['msg']['thread']['title']
            if msg['msg']['instance']['post_type'] == 'question':
                tmpl = self._("{user} deleted the question '{title}'")
            else:
                tmpl = self._("{user} deleted an answer on '{title}'")
        else:
            raise NotImplementedError("%r" % msg)

        return tmpl.format(user=user, title=title)

    def secondary_icon(self, msg, **config):
        user = None

        try:
            user = msg['msg']['agent']
        except KeyError:
            pass

        if not user:
            return ""

        return avatar_url(username=user)

    def usernames(self, msg, **config):
        users = set()

        try:
            users.add(msg['msg']['agent'])
        except KeyError:
            pass

        for user in msg['msg'].get('newly_mentioned_users', []):
            users.add(user)

        return users

    def objects(self, msg, **config):
        objs = set()

        if 'tags' in msg['msg']:
            for tag in msg['msg']['tags']:
                if not tag:
                    continue
                objs.add('tags/{tag}'.format(tag=tag))
        elif 'thread' in msg['msg']:
            for tag in msg['msg']['thread']['tagnames']:
                if not tag:
                    continue
                objs.add('tags/{tag}'.format(tag=tag))

        if 'thread' in msg['msg']:
            objs.add('threads/{pk}'.format(
                pk=msg['msg']['thread']['pk']))

        return objs

    def link(self, msg, **config):
        if 'stg' in msg['topic']:
            tmpl = "https://ask.stg.fedoraproject.org/question/{pk}/"
        else:
            tmpl = "https://ask.fedoraproject.org/question/{pk}/"

        pk = msg['msg']['topmost_post_id']
        post = ""

        if 'post' in msg['msg']:
            post = msg['msg']['post']['pk']
            #tmpl += "?answer={post}#post-id-{post}"
        elif 'instance' in msg['msg']:
            post = msg['msg']['instance']['pk']
            #tmpl += "?answer={post}#post-id-{post}"
        else:
            pass

        return tmpl.format(pk=pk, post=post)



# This file is part of fedmsg.
# Copyright (C) 2012-2016 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#           Pierre-Yves Chibon <pingou@pingoured.fr>
#
""" Tests for anitya messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestNewDistro(Base):
    """ These messages are published when a new Linux distribution is added
    to the database of `anitya <https://release-monitoring.org>`_.
    """
    expected_title = "anitya.distro.add"
    expected_subti = 'foobar added the distro named' + \
        ' "CentOS" to anitya'
    expected_link = "https://release-monitoring.org/distros"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "8d504e09d5ad44ac17e7c872c30d2e4b679a5fdcf0616e825af39466d4b14383" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['foobar'])
    expected_objects = set(['distros/CentOS'])
    msg = {
        'username': 'apache',
        'i': 4,
        'timestamp': 1467814080,
        'msg_id': u'2016-55d13473-2e48-4b6e-9542-3bc1882152ef',
        'topic': u'org.release-monitoring.prod.anitya.distro.add',
        'msg': {
            'project': None,
            'message': {
                'agent': 'foobar',
                'distro': u'CentOS'
            },
            'distro': {
                'name': 'CentOS'
            }
        }
    }


class LegacyTestNewDistro(Base):
    """ These messages are published when a new Linux distribution is added
    to the database of `anitya <https://release-monitoring.org>`_.
    """
    expected_title = "anitya.distro.add"
    expected_subti = 'olasd@debian.org added the distro named' + \
        ' "CentOS" to anitya'
    expected_link = "https://release-monitoring.org/distros"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "e202b61fca3f40ccc4c790d4ecf6ed42" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['distros/CentOS'])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1412327834,
        "msg_id": "2014-1f52337a-8dbe-48f1-baad-f2c770c60640",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.distro.add",
        "msg": {
            "project": None,
            "message": {
                "agent": "olasd@debian.org",
                "distro": "CentOS"
            },
            "distro": {
                "name": "CentOS"
            }
        }
    }


class TestEditDistro(Base):
    """ These messages are published when a Linux distribution's entry is
    edited in the `anitya <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.distro.edit"
    expected_subti = 'pingou changed a distro name' + \
        ' from "Debia" to "Debian"'
    expected_link = "https://release-monitoring.org/distros"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['distros/Debia', 'distros/Debian'])
    msg = {
        "username": "apache",
        "i": 5,
        "timestamp": 1412328814,
        "msg_id": "2014-746c39cf-9fb0-4ed1-a817-d57bc901e027",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.distro.edit",
        "msg": {
            "project": None,
            "message": {
                "new": "Debian",
                "old": "Debia",
                'agent': 'http://pingou.id.fedoraproject.org/',
            },
            "distro": {
                "name": "Debia"
            }
        }
    }


class LegacyTestEditDistro(Base):
    """ These messages are published when a Linux distribution's entry is
    edited in the `anitya <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.distro.edit"
    expected_subti = 'pingou changed a distro name' + \
        ' from "Debia" to "Debian"'
    expected_link = "https://release-monitoring.org/distros"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['distros/Debia', 'distros/Debian'])
    msg = {
        "username": "apache",
        "i": 5,
        "timestamp": 1412328814,
        "msg_id": "2014-746c39cf-9fb0-4ed1-a817-d57bc901e027",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.distro.edit",
        "msg": {
            "project": None,
            "message": {
                "new": "Debian",
                "old": "Debia",
                "agent": "pingou@fedoraproject.org"
            },
            "distro": {
                "name": "Debia"
            }
        }
    }


class TestAddProject(Base):
    """ These messages are published when someone adds a new project to
    `anitya's <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.add"
    expected_subti = 'ralph added the project "arrow" to anitya'
    expected_link = "https://release-monitoring.org/project/5314/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/arrow'])
    msg = {
        "username": "apache",
        "i": 8,
        "timestamp": 1412328939,
        "msg_id": "2014-c95e6b4c-d033-43d9-9eb1-e1f92ac87c75",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.add",
        "msg": {
            "project": {
                "regex": None,
                "name": "arrow",
                "created_on": 1412328939.0,
                "version": None,
                "versions": [],
                "version_url": None,
                "updated_on": 1412328939.0,
                "homepage": "https://pypi.python.org/pypi/arrow",
                "id": 5314,
                "backend": "PyPI"
            },
            "message": {
                "project": "arrow",
                'agent': 'http://ralph.id.fedoraproject.org/',
            },
            "distro": None
        }
    }


class LegacyTestAddProject(Base):
    """ These messages are published when someone adds a new project to
    `anitya's <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.add"
    expected_subti = 'ralph added the project "arrow" to anitya'
    expected_link = "https://release-monitoring.org/project/5314/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/arrow'])
    msg = {
        "username": "apache",
        "i": 8,
        "timestamp": 1412328939,
        "msg_id": "2014-c95e6b4c-d033-43d9-9eb1-e1f92ac87c75",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.add",
        "msg": {
            "project": {
            "regex": "",
            "name": "arrow",
            "created_on": 1412328939.0,
            "version": None,
            "version_url": "",
            "updated_on": 1412328939.0,
            "homepage": "https://pypi.python.org/pypi/arrow",
            "id": 5314,
            "backend": "pypi"
            },
            "message": {
                "project": "arrow",
                "agent": "ralph@fedoraproject.org"
            },
            "distro": None
        }
    }


class TestAddProjectTried(Base):
    """ These messages are published when someone *tries* to add a new project
    to `anitya's <https://release-monitoring.org>`_ database, but that project
    was already present.
    """
    expected_title = "anitya.project.add.tried"
    expected_subti = 'ralph tried to add the project "ansi2html" to anitya' + \
        ' (but it already exists there)'
    expected_link = "https://release-monitoring.org/project/4/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/ansi2html'])
    msg = {
        "username": "apache",
        "i": 4,
        "timestamp": 1386821688,
        "msg_id": "2013-154429ec-842e-4d7f-acae-8d7434b4cbff",
        "topic": "org.release-monitoring.prod.anitya.project.add.tried",
        "msg": {
            "project": {
                "id": 4,
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386839688.0,
                "version": None,
                "version": [],
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386839688.0,
                "packages": [],
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "agent": 'http://ralph.id.fedoraproject.org/'
            },
            "distro": None,
        }
    }


class LegacyTestAddProjectTried(Base):
    """ These messages are published when someone *tries* to add a new project
    to `anitya's <https://release-monitoring.org>`_ database, but that project
    was already present.
    """
    expected_title = "anitya.project.add.tried"
    expected_subti = 'ralph tried to add the project "ansi2html" to anitya' + \
        ' (but it already exists there)'
    expected_link = "https://release-monitoring.org/project/4/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/ansi2html'])
    msg = {
        "username": "apache",
        "i": 4,
        "timestamp": 1386821688,
        "msg_id": "2013-154429ec-842e-4d7f-acae-8d7434b4cbff",
        "topic": "org.release-monitoring.prod.anitya.project.add.tried",
        "msg": {
            "project": {
                "id": 4,
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386839688.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386839688.0,
                "packages": [],
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "agent": "rbean@redhat.com"
            },
            "distro": None,
        }
    }


class TestEditProject(Base):
    """ These messages are published when someone edits the details of a
    project in `anitya's <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.edit"
    expected_subti = 'ralph edited the following ' + \
        'fields of the "arrow" project: homepage'
    expected_link = "https://release-monitoring.org/project/5314/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/arrow'])
    msg = {
        "username": "apache",
        "i": 6,
        "timestamp": 1412329027,
        "msg_id": "2014-f25b6634-d7b9-4f2d-9f93-23ef73034024",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.edit",
        "msg": {
            "project": {
                "regex": None,
                "name": "arrow",
                "created_on": 1412328939.0,
                "version": None,
                "versions": [],
                "version_url": None,
                "updated_on": 1412328939.0,
                "homepage": "http://crsmithdev.com/arrow",
                "id": 5314,
                "backend": "PyPI"
            },
            "message": {
                "project": "arrow",
                "fields": [
                    "homepage"
                ],
                "agent": "http://ralph.id.fedoraproject.org/"
            },
            "distro": None
        }
    }


class LegacyTestEditProject(Base):
    """ These messages are published when someone edits the details of a
    project in `anitya's <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.edit"
    expected_subti = 'ralph edited the following ' + \
        'fields of the "arrow" project: homepage'
    expected_link = "https://release-monitoring.org/project/5314/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/arrow'])
    msg = {
        "username": "apache",
        "i": 6,
        "timestamp": 1412329027,
        "msg_id": "2014-f25b6634-d7b9-4f2d-9f93-23ef73034024",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.edit",
        "msg": {
            "project": {
                "regex": "",
                "name": "arrow",
                "created_on": 1412328939.0,
                "version": None,
                "version_url": "",
                "updated_on": 1412328939.0,
                "homepage": "http://crsmithdev.com/arrow",
                "id": 5314,
                "backend": "pypi"
            },
            "message": {
                "project": "arrow",
                "fields": [
                    "homepage"
                ],
                "agent": "ralph@fedoraproject.org"
            },
            "distro": None
        }
    }


class TestRemoveProject(Base):
    """ These messages are published when someone *removes* a project from
    `anitya's <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.remove"
    expected_subti = 'ralph deleted the "guake" project'
    expected_link = "https://release-monitoring.org/project/5311/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/guake'])
    msg = {
        "username": "apache",
        "i": 11,
        "timestamp": 1412331340,
        "msg_id": "2014-94864be5-f649-4b3f-8694-32f238ac7174",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.remove",
        "msg": {
            "project": {
                "regex": None,
                "name": "guake",
                "created_on": 1412237149.0,
                "version": "0.5.0",
                "versions": [],
                "version_url": "guake/guake",
                "updated_on": 1412237231.0,
                "homepage": "http://guake.org",
                "id": 5311,
                "backend": "GitHub"
            },
            "message": {
                "project": "guake",
                "agent": "http://ralph.id.fedoraproject.org/"
            },
            "distro": None
        }
    }


class LegacyTestRemoveProject(Base):
    """ These messages are published when someone *removes* a project from
    `anitya's <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.remove"
    expected_subti = 'ralph deleted the "guake" project'
    expected_link = "https://release-monitoring.org/project/5311/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/guake'])
    msg = {
        "username": "apache",
        "i": 11,
        "timestamp": 1412331340,
        "msg_id": "2014-94864be5-f649-4b3f-8694-32f238ac7174",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.remove",
        "msg": {
            "project": {
                "regex": "",
                "name": "guake",
                "created_on": 1412237149.0,
                "version": "0.5.0",
                "version_url": "guake/guake",
                "updated_on": 1412237231.0,
                "homepage": "http://guake.org",
                "id": 5311,
                "backend": "Github"
            },
            "message": {
                "project": "guake",
                "agent": "ralph@fedoraproject.org"
            },
            "distro": None
        }
    }


class TestNewMappingProject(Base):
    """ These messages are published when someone maps an upstream project to a
    package name in a particular distribution (in the `anitya
    <https://release-monitoring.org>`_ database...)
    """
    expected_title = "anitya.project.map.new"
    expected_subti = 'ralph mapped the name of "arrow"' + \
        ' in Fedora to "python-arrow"'
    expected_link = "https://release-monitoring.org/project/5314/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set(['python-arrow'])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'projects/arrow',
        'mappings/Fedora/python-arrow',
        'distros/Fedora',
    ])
    msg = {
        "username": "apache",
        "i": 7,
        "timestamp": 1412329216,
        "msg_id": "2014-9ebf960a-115e-4568-a615-34cc7d8d462e",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.map.new",
        "msg": {
            "project": {
                "regex": None,
                "name": "arrow",
                "created_on": 1412328939.0,
                "version": None,
                "versions": [],
                "version_url": None,
                "updated_on": 1412329027.0,
                "homepage": "http://crsmithdev.com/arrow",
                "id": 5314,
                "backend": "PyPI"
            },
            "message": {
                "project": "arrow",
                "new": "python-arrow",
                "agent": "http://ralph.id.fedoraproject.org/",
                "distro": "Fedora"
            },
            "distro": {
                "name": "Fedora"
            }
        }
    }


class LegacyTestNewMappingProject(Base):
    """ These messages are published when someone maps an upstream project to a
    package name in a particular distribution (in the `anitya
    <https://release-monitoring.org>`_ database...)
    """
    expected_title = "anitya.project.map.new"
    expected_subti = 'ralph mapped the name of "arrow"' + \
        ' in Fedora to "python-arrow"'
    expected_link = "https://release-monitoring.org/project/5314/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set(['python-arrow'])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'projects/arrow',
        'mappings/Fedora/python-arrow',
        'distros/Fedora',
    ])
    msg = {
        "username": "apache",
        "i": 7,
        "timestamp": 1412329216,
        "msg_id": "2014-9ebf960a-115e-4568-a615-34cc7d8d462e",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.map.new",
        "msg": {
            "project": {
            "regex": "",
            "name": "arrow",
            "created_on": 1412328939.0,
            "version": None,
            "version_url": "",
            "updated_on": 1412329027.0,
            "homepage": "http://crsmithdev.com/arrow",
            "id": 5314,
            "backend": "pypi"
            },
            "message": {
                "project": "arrow",
                "new": "python-arrow",
                "agent": "ralph@fedoraproject.org",
                "distro": "Fedora"
            },
            "distro": {
                "name": "Fedora"
            }
        }
    }


class TestUpdatedMappingProject(Base):
    """ These messages are published when someone updates the mapping between
    an upstream project and a package name in a particular distribution
    (in the `anitya <https://release-monitoring.org>`_ database...)
    """
    expected_title = "anitya.project.map.update"
    expected_subti = 'ralph updated the name of ' + \
        '"guake" in "Fedora" from "guake2" to "guake"'
    expected_link = "https://release-monitoring.org/project/5311/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([
        'guake',
        'guake2',
    ])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'projects/guake',
        'mappings/Fedora/guake2',
        'mappings/Fedora/guake',
        'distros/Fedora',
    ])
    msg = {
        "username": "apache",
        "i": 8,
        "timestamp": 1412329667,
        "msg_id": "2014-5bd228ac-4ba1-452f-b6d2-b4df53c9af14",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.map.update",
        "msg": {
            "project": {
                "regex": None,
                "name": "guake",
                "created_on": 1412237149.0,
                "version": "0.5.0",
                "versions": [],
                "version_url": "guake/guake",
                "updated_on": 1412237231.0,
                "homepage": "http://guake.org",
                "id": 5311,
                "backend": "Github"
            },
            "message": {
                "edited": [
                    "package_name"
                ],
                "agent": "http://ralph.id.fedoraproject.org/",
                "project": "guake",
                "new": "guake",
                "prev": "guake2",
                "distro": "Fedora"
            },
            "distro": {
                "name": "Fedora"
            }
        }
    }


class LegacyTestUpdatedMappingProject(Base):
    """ These messages are published when someone updates the mapping between
    an upstream project and a package name in a particular distribution
    (in the `anitya <https://release-monitoring.org>`_ database...)
    """
    expected_title = "anitya.project.map.update"
    expected_subti = 'ralph updated the name of ' + \
        '"guake" in "Fedora" from "guake2" to "guake"'
    expected_link = "https://release-monitoring.org/project/5311/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_packages = set([
        'guake',
        'guake2',
    ])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'projects/guake',
        'mappings/Fedora/guake2',
        'mappings/Fedora/guake',
        'distros/Fedora',
    ])
    msg = {
        "username": "apache",
        "i": 8,
        "timestamp": 1412329667,
        "msg_id": "2014-5bd228ac-4ba1-452f-b6d2-b4df53c9af14",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.map.update",
        "msg": {
            "project": {
                "regex": "",
                "name": "guake",
                "created_on": 1412237149.0,
                "version": "0.5.0",
                "version_url": "guake/guake",
                "updated_on": 1412237231.0,
                "homepage": "http://guake.org",
                "id": 5311,
                "backend": "Github"
            },
            "message": {
                "edited": [
                    "package_name"
                ],
                "agent": "ralph@fedoraproject.org",
                "project": "guake",
                "new": "guake",
                "prev": "guake2",
                "distro": "Fedora"
            },
            "distro": {
                "name": "Fedora"
            }
        }
    }


class TestFirstNewUpstreamVersionLegacy(Base):
    expected_title = "anitya.project.version.update"
    expected_subti = ('A new version of "Accanthis-Std" has been detected:  "20101124", '
        'packaged as "adf-accanthis-fonts"')
    expected_link = "https://release-monitoring.org/project/22/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    #expected_secondary_icon = None
    expected_packages = set([
        'adf-accanthis-fonts',
    ])
    expected_usernames = set(['anitya'])
    expected_objects = set([
        'projects/Accanthis-Std',
    ])
    msg = {
        "username": "anitya",
        "i": 1,
        "timestamp": 1412790624,
        "msg_id": "2014-3120499f-993b-4526-81c7-8abc39207f9e",
        "topic": "org.release-monitoring.prod.anitya.project.version.update",
        "msg": {
            "project": {
                "regex": "",
                "name": "Accanthis-Std",
                "created_on": 1412174945.0,
                "version": "20101124",
                "version_url": "http://arkandis.tuxfamily.org/adffonts.html",
                "updated_on": 1412697102.0,
                "homepage": "http://arkandis.tuxfamily.org/adffonts.html",
                "id": 22,
                "backend": "custom"
            },
            "packages": [
                {
                    "package_name": "adf-accanthis-fonts",
                    "distro": "Fedora"
                }
            ],
            "old_version": "",
            "upstream_version": "20101124",
            "versions": [
                "20101124"
            ]
        }
    }


class TestNewUpstreamVersion(Base):
    """ The purpose of anitya is to monitor upstream projects and to
    try and detect when they release new tarballs.

    *These* messages are the ones that get published when a tarball is found
    that is newer than the one last seen in the `anitya
    <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.version.update"
    expected_subti = ('A new version of "2ping" has been detected:  "2.1.1" '
        'newer than "2.1.0", packaged as "2ping"')
    expected_link = "https://release-monitoring.org/project/2/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    #expected_secondary_icon = None
    expected_packages = set([
        '2ping',
    ])
    expected_usernames = set(['anitya'])
    expected_objects = set([
        'projects/2ping',
    ])
    msg = {
        "username": "fedmsg",
        "i": 1,
        "timestamp": 1412234961,
        "msg_id": "2014-f4dfc3e4-8909-45d7-b929-1862efb373cf",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.version.update",
        "msg": {
            "project": {
                "regex": None,
                "name": "2ping",
                "created_on": 1412174944.0,
                "version": "2.1.1",
                "version_url": "http://www.finnie.org/software/2ping/",
                "updated_on": 1412179539.0,
                "homepage": "http://www.finnie.org/software/2ping/",
                "id": 2,
                "backend": "custom"
            },
            "message": {
                "versions": [
                    "2.1.1"
                ],
                "old_version": "2.1.0",
                "upstream_version": "2.1.1",
                "project": {
                    "regex": None,
                    "name": "2ping",
                    "created_on": 1412174944.0,
                    "version": "2.1.1",
                    "version_url": "http://www.finnie.org/software/2ping/",
                    "updated_on": 1412179539.0,
                    "homepage": "http://www.finnie.org/software/2ping/",
                    "id": 2,
                    "backend": "custom"
                },
                "agent": "anitya",
                "packages": [
                    {
                        "package_name": "2ping",
                        "distro": "Fedora"
                    }
                ]
            },
            "distro": None
          }
    }


class TestOddNewUpstreamVersion(Base):
    """ The purpose of anitya is to monitor upstream projects and to
    try and detect when they release new tarballs.

    *These* messages are the ones that get published when a tarball is found
    that is older than the one last seen in the `anitya
    <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.version.update"
    expected_subti = ('A new version of "2ping" has been detected:  '
        '"2.1.0", packaged as "2ping"')
    expected_link = "https://release-monitoring.org/project/2/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    #expected_secondary_icon = None
    expected_packages = set([
        '2ping',
    ])
    expected_usernames = set(['anitya'])
    expected_objects = set([
        'projects/2ping',
    ])
    msg = {
        "username": "fedmsg",
        "i": 1,
        "timestamp": 1412234961,
        "msg_id": "2014-f4dfc3e4-8909-45d7-b929-1862efb373cf",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.version.update",
        "msg": {
            "project": {
                "regex": None,
                "name": "2ping",
                "created_on": 1412174944.0,
                "version": "2.1.1",
                "version_url": "http://www.finnie.org/software/2ping/",
                "updated_on": 1412179539.0,
                "homepage": "http://www.finnie.org/software/2ping/",
                "id": 2,
                "backend": "custom"
            },
            "message": {
                "versions": [
                    "2.1.1"
                ],
                "old_version": "2.1.1",
                "upstream_version": "2.1.0",
                "project": {
                    "regex": None,
                    "name": "2ping",
                    "created_on": 1412174944.0,
                    "version": "2.1.1",
                    "version_url": "http://www.finnie.org/software/2ping/",
                    "updated_on": 1412179539.0,
                    "homepage": "http://www.finnie.org/software/2ping/",
                    "id": 2,
                    "backend": "custom"
                },
                "odd_change": True,
                "agent": "anitya",
                "packages": [
                    {
                        "package_name": "2ping",
                        "distro": "Fedora"
                    }
                ]
            },
            "distro": None
          }
    }


class TestNewUpstreamVersionMutiMap(Base):
    """ The purpose of anitya is to monitor upstream projects and to
    try and detect when they release new tarballs.

    *These* messages are the ones that get published when a tarball is found
    that is newer than the one last seen in the `anitya
    <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.version.update"
    expected_subti = ('A new version of "SQLAlchemy" has been detected:  '
        '"1.0.0b1" newer than "0.9.9", packaged as '
        '"python-sqlalchemy and python-sqlalchemy0.5"')
    expected_link = "https://release-monitoring.org/project/4034/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    #expected_secondary_icon = None
    expected_packages = set(['python-sqlalchemy', 'python-sqlalchemy0.5'])
    expected_usernames = set(['anitya'])
    expected_objects = set([
        'projects/SQLAlchemy',
    ])
    msg = {
      "source_name": "datanommer",
      "i": 17,
      "timestamp": 1426296431.0,
      "msg_id": "2015-b2e3fab5-12a6-47b0-9ffc-c21d5789a2d0",
      "topic": "org.release-monitoring.prod.anitya.project.version.update",
      "source_version": "0.6.4",
      "msg": {
            "project": {
            "regex": None,
            "name": "SQLAlchemy",
            "versions": [
                "1.0.0b1",
                "0.9.9",
                "0.9.8",
                "0.9.7"
                ],
            "created_on": 1412175085.0,
            "version": "1.0.0b1",
            "version_url": None,
            "updated_on": 1426296430.0,
            "homepage": "https://pypi.python.org/pypi/SQLAlchemy",
            "id": 4034,
            "backend": "pypi"
            },
            "message": {
                "versions": [
                    "1.0.0b1",
                    "0.9.9",
                    "0.9.8",
                    "0.9.7"
                    ],
                "old_version": "0.9.9",
                "upstream_version": "1.0.0b1",
                "project": {
                    "regex": None,
                    "name": "SQLAlchemy",
                    "versions": [
                    "1.0.0b1",
                    "0.9.9",
                    "0.9.8",
                    "0.9.7"
                    ],
                "created_on": 1412175085.0,
                "version": "1.0.0b1",
                "version_url": None,
                "updated_on": 1426167440.0,
                "homepage": "https://pypi.python.org/pypi/SQLAlchemy",
                "id": 4034,
                "backend": "pypi"
                 },
                "agent": "anitya",
                "packages": [
                    {
                        "package_name": "python-sqlalchemy",
                        "distro": "Fedora"
                    },
                    {
                        "package_name": "python-sqlalchemy0.5",
                        "distro": "Fedora"
                    }
                ]
            },
            "distro": None
        }
    }


class TestFirstNewUpstreamVersion(Base):
    """ The purpose of anitya is to monitor upstream projects and to
    try and detect when they release new tarballs.

    *This* message is an example of what gets published when **the first
    tarball of a project is ever seen** by `anitya
    <https://release-monitoring.org>`_.
    """
    expected_title = "anitya.project.version.update"
    expected_subti = ('A new version of "2ping" has been detected:  "2.1.1", '
        'packaged as "2ping"')
    expected_link = "https://release-monitoring.org/project/2/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    #expected_secondary_icon = None
    expected_packages = set([
        '2ping',
    ])
    expected_usernames = set(['anitya'])
    expected_objects = set([
        'projects/2ping',
    ])
    msg = {
        "username": "fedmsg",
        "i": 1,
        "timestamp": 1412234961,
        "msg_id": "2014-f4dfc3e4-8909-45d7-b929-1862efb373cf",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.version.update",
        "msg": {
            "project": {
                "regex": None,
                "name": "2ping",
                "created_on": 1412174944.0,
                "version": "2.1.1",
                "version_url": "http://www.finnie.org/software/2ping/",
                "updated_on": 1412179539.0,
                "homepage": "http://www.finnie.org/software/2ping/",
                "id": 2,
                "backend": "custom"
            },
            "message": {
                "versions": [
                    "2.1.1"
                ],
                "old_version": "",
                "upstream_version": "2.1.1",
                "project": {
                    "regex": None,
                    "name": "2ping",
                    "created_on": 1412174944.0,
                    "version": "2.1.1",
                    "version_url": "http://www.finnie.org/software/2ping/",
                    "updated_on": 1412179539.0,
                    "homepage": "http://www.finnie.org/software/2ping/",
                    "id": 2,
                    "backend": "custom"
                },
                "agent": "anitya",
                "packages": [
                    {
                        "package_name": "2ping",
                        "distro": "Fedora"
                    }
                ]
            },
            "distro": None
          }
    }


class TestRemoveMappingProject(Base):
    """ These messages are published when someone *removes* a mapping
    between an upstream project and a package name in a particular
    distribution (in the `anitya <https://release-monitoring.org>`_
    database...)
    """
    expected_title = "anitya.project.map.remove"
    expected_subti = 'pingou deleted the mapping of ' + \
        '"guake" project on "Fedora"'
    expected_link = "https://release-monitoring.org/project/5311/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/guake'])
    msg = {
        "username": "apache",
        "i": 10,
        "timestamp": 1412330764,
        "msg_id": "2014-1d43e5ab-e398-4007-8269-26b4f209d55b",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.map.remove",
        "msg": {
            "project": {
                "regex": "",
                "name": "guake",
                "created_on": 1412237149.0,
                "version": "0.5.0",
                "version_url": "guake/guake",
                "updated_on": 1412237231.0,
                "homepage": "http://guake.org",
                "id": 5311,
                "backend": "Github"
            },
            "message": {
                "project": "guake",
                "agent": "http://pingou.id.fedoraproject.org/",
                "distro": "Fedora"
            },
            "distro": None
        }
    }


class LegacyTestRemoveMappingProject(Base):
    """ These messages are published when someone *removes* a mapping
    between an upstream project and a package name in a particular
    distribution (in the `anitya <https://release-monitoring.org>`_
    database...)
    """
    expected_title = "anitya.project.map.remove"
    expected_subti = 'pingou deleted the mapping of ' + \
        '"guake" project on "Fedora"'
    expected_link = "https://release-monitoring.org/project/5311/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/guake'])
    msg = {
        "username": "apache",
        "i": 10,
        "timestamp": 1412330764,
        "msg_id": "2014-1d43e5ab-e398-4007-8269-26b4f209d55b",
        "crypto": "x509",
        "topic": "org.release-monitoring.prod.anitya.project.map.remove",
        "msg": {
            "project": {
                "regex": "",
                "name": "guake",
                "created_on": 1412237149.0,
                "version": "0.5.0",
                "version_url": "guake/guake",
                "updated_on": 1412237231.0,
                "homepage": "http://guake.org",
                "id": 5311,
                "backend": "Github"
            },
            "message": {
                "project": "guake",
                "agent": "pingou@fedoraproject.org",
                "distro": "Fedora"
            },
            "distro": None
        }
    }


class TestRemoveVersionProject(Base):
    """ These messages are published when an admin *removes* a version
    from a particular project (in the `anitya <https://release-monitoring.org>`_
    database...)
    """
    expected_title = "anitya.project.version.remove"
    expected_subti = 'pingou deleted the version 0.7.1.1 of "3proxy"'
    expected_link = "https://release-monitoring.org/project/3/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/3proxy'])
    msg = {
        "username": "pingou",
        "i": 1,
        "timestamp": 1415118527,
        "msg_id": "2014-7926f638-1d0c-470c-b589-de84f5d34fad",
        "topic": "org.release-monitoring.prod.anitya.project.version.remove",
        "msg": {
            "project": {
                "regex": None,
                "name": "3proxy",
                "created_on": 1409917223.0,
                "version": "0.7.1.1",
                "versions": [],
                "version_url": "http://www.3proxy.ru/download/",
                "updated_on": 1412690620.0,
                "homepage": "http://www.3proxy.ru/download/",
                "id": 3,
                "backend": "custom"
            },
            "message": {
                "project": "3proxy",
                "version": "0.7.1.1",
                "agent": "http://pingou.id.fedoraproject.org/"
            },
            "distro": None
        }
    }


class LegacyTestRemoveVersionProject(Base):
    """ These messages are published when an admin *removes* a version
    from a particular project (in the `anitya <https://release-monitoring.org>`_
    database...)
    """
    expected_title = "anitya.project.version.remove"
    expected_subti = 'pingou deleted the version 0.7.1.1 of "3proxy"'
    expected_link = "https://release-monitoring.org/project/3/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/3proxy'])
    msg = {
        "username": "pingou",
        "i": 1,
        "timestamp": 1415118527,
        "msg_id": "2014-7926f638-1d0c-470c-b589-de84f5d34fad",
        "topic": "org.release-monitoring.prod.anitya.project.version.remove",
        "msg": {
            "project": {
                "regex": None,
                "name": "3proxy",
                "created_on": 1409917223.0,
                "version": "0.7.1.1",
                "version_url": "http://www.3proxy.ru/download/",
                "updated_on": 1412690620.0,
                "homepage": "http://www.3proxy.ru/download/",
                "id": 3,
                "backend": "custom"
            },
            "message": {
                "project": "3proxy",
                "version": "0.7.1.1",
                "agent": "pingou@fedoraproject.org"
            },
            "distro": None
        }
    }


class TestRemoveDistro(Base):
    """ These messages are published when an admin *removes* a distribution
    (in the `anitya <https://release-monitoring.org>`_ database...)
    """
    expected_title = "anitya.distro.remove"
    expected_subti = 'pingou deleted the distro "Arch"'
    expected_link = "https://release-monitoring.org/distros"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['distros/Arch'])
    msg = {
        "username": "pierrey",
        "i": 4,
        "timestamp": 1418315977,
        "msg_id": "2014-745132e5-17eb-4785-8663-fa89801a08ee",
        "topic": "org.release-monitoring.dev.anitya.distro.remove",
        "msg": {
            "project": None,
            "message": {
                "agent": "http://pingou.id.fedoraproject.org/",
                "distro": "Arch"
            },
            "distro": {
                "name": "Arch"
            }
        }
    }


class LegacyTestRemoveDistro(Base):
    """ These messages are published when an admin *removes* a distribution
    (in the `anitya <https://release-monitoring.org>`_ database...)
    """
    expected_title = "anitya.distro.remove"
    expected_subti = 'pingou deleted the distro "Arch"'
    expected_link = "https://release-monitoring.org/distros"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['distros/Arch'])
    msg = {
        "username": "pierrey",
        "i": 4,
        "timestamp": 1418315977,
        "msg_id": "2014-745132e5-17eb-4785-8663-fa89801a08ee",
        "topic": "org.release-monitoring.dev.anitya.distro.remove",
        "msg": {
            "project": None,
            "message": {
                "agent": "pingou@fedoraproject.org",
                "distro": "Arch"
            },
            "distro": {
                "name": "Arch"
            }
        }
    }


class TestProjectFlag(Base):
    """ These messages are published when an user *flags* a project
    in the `anitya <https://release-monitoring.org>`_ database to ask an
    admin to do something on a project.
    """
    expected_title = "anitya.project.flag"
    expected_subti = 'pingou flagged the project "generic-colouriser"'
    expected_link = "https://release-monitoring.org/project/5777/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/generic-colouriser'])
    msg = {
          "source_name": "datanommer",
          "i": 4,
          "timestamp": 1445900487.0,
          "msg_id": "2015-7b2fca29-7409-43f2-949f-c688437ba5d4",
          "topic": "org.release-monitoring.prod.anitya.project.flag",
          "source_version": "0.6.5",
          "msg": {
            "project": {
              "regex": None,
              "name": "generic-colouriser",
              "versions": [
                "v1.9",
                "v1.7"
              ],
              "created_on": 1425912743.0,
              "version": "v1.9",
              "version_url": "garabik/grc",
              "updated_on": 1427848695.0,
              "homepage": "http://kassiopeia.juls.savba.sk/~garabik/software/grc.html",
              "id": 5777,
              "backend": "Github"
            },
            "message": {
              "project": "generic-colouriser",
              "reason": "Delete in favor of the correctly configured 7894.",
              "agent": "http://pingou.id.fedoraproject.org/"
            },
            "distro": None
          }
        }


class LegacyTestProjectFlag(Base):
    """ These messages are published when an user *flags* a project
    in the `anitya <https://release-monitoring.org>`_ database to ask an
    admin to do something on a project.
    """
    expected_title = "anitya.project.flag"
    expected_subti = 'pingou flagged the project "generic-colouriser"'
    expected_link = "https://release-monitoring.org/project/5777/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/generic-colouriser'])
    msg = {
          "source_name": "datanommer",
          "i": 4,
          "timestamp": 1445900487.0,
          "msg_id": "2015-7b2fca29-7409-43f2-949f-c688437ba5d4",
          "topic": "org.release-monitoring.prod.anitya.project.flag",
          "source_version": "0.6.5",
          "msg": {
            "project": {
              "regex": "",
              "name": "generic-colouriser",
              "versions": [
                "v1.9",
                "v1.7"
              ],
              "created_on": 1425912743.0,
              "version": "v1.9",
              "version_url": "garabik/grc",
              "updated_on": 1427848695.0,
              "homepage": "http://kassiopeia.juls.savba.sk/~garabik/software/grc.html",
              "id": 5777,
              "backend": "Github"
            },
            "message": {
              "project": "generic-colouriser",
              "reason": "Delete in favor of the correctly configured 7894.",
              "agent": "pingou@fedoraproject.org"
            },
            "distro": None
          }
        }


class TestProjectFlag_WithPackages(Base):
    """ These messages are published when an user *flags* a project
    in the `anitya <https://release-monitoring.org>`_ database to ask an
    admin to do something on a project.
    """
    expected_title = "anitya.project.flag"
    expected_subti = 'pingou flagged the project "generic-colouriser"'
    expected_link = "https://release-monitoring.org/project/5777/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set(['generic-colouriser'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/generic-colouriser'])
    msg = {
          "source_name": "datanommer",
          "i": 4,
          "timestamp": 1445900487.0,
          "msg_id": "2015-7b2fca29-7409-43f2-949f-c688437ba5d4",
          "topic": "org.release-monitoring.prod.anitya.project.flag",
          "source_version": "0.6.5",
          "msg": {
            "project": {
              "regex": None,
              "name": "generic-colouriser",
              "versions": [
                "v1.9",
                "v1.7"
              ],
              "created_on": 1425912743.0,
              "version": "v1.9",
              "version_url": "garabik/grc",
              "updated_on": 1427848695.0,
              "homepage": "http://kassiopeia.juls.savba.sk/~garabik/software/grc.html",
              "id": 5777,
              "backend": "GitHub"
            },
            "message": {
              "project": "generic-colouriser",
              "reason": "Delete in favor of the correctly configured 7894.",
              "agent": "http://pingou.id.fedoraproject.org/"
            },
            "distro": None,
            "packages": [
                {"package_name": "generic-colouriser", "distro": "Fedora"}
            ]
          }
        }


class LegacyTestProjectFlag_WithPackages(Base):
    """ These messages are published when an user *flags* a project
    in the `anitya <https://release-monitoring.org>`_ database to ask an
    admin to do something on a project.
    """
    expected_title = "anitya.project.flag"
    expected_subti = 'pingou flagged the project "generic-colouriser"'
    expected_link = "https://release-monitoring.org/project/5777/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set(['generic-colouriser'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['projects/generic-colouriser'])
    msg = {
          "source_name": "datanommer",
          "i": 4,
          "timestamp": 1445900487.0,
          "msg_id": "2015-7b2fca29-7409-43f2-949f-c688437ba5d4",
          "topic": "org.release-monitoring.prod.anitya.project.flag",
          "source_version": "0.6.5",
          "msg": {
            "project": {
              "regex": "",
              "name": "generic-colouriser",
              "versions": [
                "v1.9",
                "v1.7"
              ],
              "created_on": 1425912743.0,
              "version": "v1.9",
              "version_url": "garabik/grc",
              "updated_on": 1427848695.0,
              "homepage": "http://kassiopeia.juls.savba.sk/~garabik/software/grc.html",
              "id": 5777,
              "backend": "Github"
            },
            "message": {
              "project": "generic-colouriser",
              "reason": "Delete in favor of the correctly configured 7894.",
              "agent": "pingou@fedoraproject.org"
            },
            "distro": None,
            "packages": [
                {"package_name": "generic-colouriser", "distro": "Fedora"}
            ]
          }
        }


class TestFlagSet(Base):
    """ These messages are published when an user change the status of a
    flag in the `anitya <https://release-monitoring.org>`_ database.
    """
    expected_title = "anitya.project.flag.set"
    expected_subti = 'pingou closed flag "184"'
    expected_link = "https://release-monitoring.org/"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['flag/184'])
    msg = {
          "source_name": "datanommer",
          "i": 4,
          "timestamp": 1468318351.0,
          "msg_id": "2016-fc73c74c-fc5a-4f65-8ae9-87145bae82e4",
          "topic": "org.release-monitoring.prod.anitya.project.flag.set",
          "source_version": "0.6.5",
          "msg": {
            "project": None,
            "message": {
              "flag": 184,
              "state": "closed",
              "agent": "pingou@fedoraproject.org"
            },
            "distro": None
          }
        }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
#
# Authors:  Hammad Haleem <hammadhaleem@gmail.com>
'''Preprocessor for fedora college'''

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class FedoraCollegeProcessor(BaseProcessor):
    __name__ = "FedoraCollege"
    __description__ = "The virtual Classroom for new fedora contributors"
    __obj__ = "Delivering video tutorials"

    '''
    currently not in production
    __link__ = "https://ask.fedoraproject.org"
    __docs__ = "https://askbot.com"
    __icon__ = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    '''

    def subtitle(self, msg, **config):

        username = msg['msg']['username']
        title = msg['msg']['title']

        if "fedoracollege.media.upload" in msg['topic']:
            tmpl = self._(
                '{username} uploaded a new file of type "{title}"')
        elif "fedoracollege.content.added" in msg['topic']:
            tmpl = self._(
                '{username} Created new content titled "{title}"')
        elif "fedoracollege.content.edit" in msg['topic']:
            tmpl = self._(
                '{username} Edited content titled "{title}"')
        else:
            raise NotImplementedError("%r" % msg)

        return tmpl.format(username=username, title=title)

    def secondary_icon(self, msg, **config):
        user = None

        try:
            user = msg['msg']['username']
        except KeyError:
            pass

        if not user:
            return ""

        return avatar_url(username=user)

    def usernames(self, msg, **config):
        if 'username' in msg['msg']:
            return set([msg['msg']['username']])

        return set()

    def link(self, msg, **config):
        return msg['msg']['link']



from fedmsg.tests.test_meta import Base


def add_doc(objects):
    """ Given a dict of names and classes, reattach the docstring as 'doc'.

    The core fedmsg lib uses that 'doc' attribute to build its own docs for
    http://fedmsg.com/

    This is needed as a workaround for this code:
    https://github.com/fedora-infra/fedmsg/blob/develop/fedmsg/doc_utilities.py
    """

    for k, v in objects.items():
        if 'Test' in k and issubclass(v, Base):
            v.doc = v.__doc__



# This file is part of fedmsg.
# Copyright (C) 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for compose messages """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestTwoWeekAtomicBegin(Base):
    """ As one of the final steps in the `two week atomic
    <https://fedoraproject.org/wiki/Changes/Two_Week_Atomic>`_ process, there
    is a release engineering script (the green box in `this diagram
    <https://fedoraproject.org/wiki/File:TwoWeekAtomic-Overview-v2.png>`_) that
    determines if the builds of Fedora Atomic Host are fit for release.

    If it determines that the builds are ready (usually by cross-referencing
    test results from `autocloud <https://apps.fedoraproject.org/autocloud>`_),
    then it publishes one of these messages, indicating that the website should
    be updated to link to the new builds.
    """

    expected_title = "releng.atomic.twoweek.begin"
    expected_subti = "Release engineering scripts started evaluating a " + \
        "new set of builds for a Fedora 22 Atomic Host release"
    expected_icon = "https://apps.fedoraproject.org/img/icons/atomic.png"
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/atomic.png"
    expected_link = "https://download.fedoraproject.org/pub/alt/atomic/stable/"
    expected_objects = set(['22/atomic_qcow2', '22/atomic_raw'])

    msg = {
        "timestamp": 1444939709.0,
        "topic": "org.fedoraproject.prod.releng.atomic.twoweek.begin",
        "msg": {
            "atomic_qcow2": {
                "release": "22",
                "image_url": "https://kojipkgs.fedoraproject.org//work/tasks/"
                "3513/11453513/Fedora-Cloud-Atomic-22-20151015.x86_64.qcow2",
                "image_name": "Fedora-Cloud-Atomic-22-20151015.x86_64.qcow2",
                "name": "Fedora-Cloud-Atomic"
            },
            "atomic_raw": {
                "release": "22",
                "image_url": "https://kojipkgs.fedoraproject.org//work/tasks/"
                "3513/11453513/Fedora-Cloud-Atomic-22-20151015.x86_64.raw.xz",
                "image_name": "Fedora-Cloud-Atomic-22-20151015.x86_64.raw.xz",
                "name": "Fedora-Cloud-Atomic-Raw"
            }
        }
    }


class TestTwoWeekAtomicComplete(Base):
    """ As one of the final steps in the `two week atomic
    <https://fedoraproject.org/wiki/Changes/Two_Week_Atomic>`_ process, there
    is a release engineering script (the green box in `this diagram
    <https://fedoraproject.org/wiki/File:TwoWeekAtomic-Overview-v2.png>`_) that
    determines if the builds of Fedora Atomic Host are fit for release.

    If it determines that the builds are ready (usually by cross-referencing
    test results from `autocloud <https://apps.fedoraproject.org/autocloud>`_),
    then it publishes one of these messages, indicating that the website should
    be updated to link to the new builds.
    """

    expected_title = "releng.atomic.twoweek.complete"
    expected_subti = "A new release of Fedora 22 Atomic Host is ready"
    expected_icon = "https://apps.fedoraproject.org/img/icons/atomic.png"
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/atomic.png"
    expected_link = "https://download.fedoraproject.org/pub/alt/atomic/stable/"
    expected_objects = set(['22/atomic_qcow2', '22/atomic_raw'])

    msg = {
        "timestamp": 1444939709.0,
        "topic": "org.fedoraproject.prod.releng.atomic.twoweek.complete",
        "msg": {
            "atomic_qcow2": {
                "release": "22",
                "image_url": "https://kojipkgs.fedoraproject.org//work/tasks/"
                "3513/11453513/Fedora-Cloud-Atomic-22-20151015.x86_64.qcow2",
                "image_name": "Fedora-Cloud-Atomic-22-20151015.x86_64.qcow2",
                "name": "Fedora-Cloud-Atomic"
            },
            "atomic_raw": {
                "release": "22",
                "image_url": "https://kojipkgs.fedoraproject.org//work/tasks/"
                "3513/11453513/Fedora-Cloud-Atomic-22-20151015.x86_64.raw.xz",
                "image_name": "Fedora-Cloud-Atomic-22-20151015.x86_64.raw.xz",
                "name": "Fedora-Cloud-Atomic-Raw"
            }
        }
    }


add_doc(locals())



# This file is part of fedmsg.
# Copyright (C) 2017 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for waiverdb messages """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestNewWaiver(Base):
    """ `WaiverDB
    <http://fedoraproject.org/wiki/Infrastructure/Factory2/Focus/WaiverDB>`_ is
    a service that allows humans to override test failures and influence gating
    decisions made by `Greenwave
    <http://fedoraproject.org/wiki/Infrastructure/Factory2/Focus/Greenwave>`_.

    It publishes messages like this whenever someone **records a new waiver**.
    """

    expected_title = "waiverdb.waiver.new"
    expected_subti = "ralph waived result 123 (fedora-26): \"This is fine.\""
    expected_link = (
        'https://waiverdb-web-waiverdb.app.os.fedoraproject.org/'
        'api/v1.0/waivers/8')
    expected_icon = 'https://apps.fedoraproject.org/img/icons/waiverdb.png'
    expected_secondary_icon = (
        'https://seccdn.libravatar.org/avatar/'
        '9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c'
        '?s=64&d=retro')
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    msg = {
        "username": "openshift",
        "i": 2,
        "timestamp": 1510020810.0,
        "msg_id": "2017-b173d0d1-a119-40e6-9e85-c6178468d9ff",
        "topic": "org.fedoraproject.stg.waiverdb.waiver.new",
        "headers": {},
        "msg": {
          "comment": "This is fine.",
          "username": "ralph",
          "waived": True,
          "timestamp": "2017-11-07T02:13:30.466388",
          "product_version": "fedora-26",
          "result_id": 123,
          "id": 8
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2017 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#

from six.moves.urllib.parse import urlencode
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class GreenwaveProcessor(BaseProcessor):
    __name__ = "greenwave"
    __description__ = "Greenwave Gating Service"
    __link__ = "greenwave-web-greenwave.app.os.fedoraproject.org/api/v1.0/policies"
    __docs__ = "https://fedoraproject.org/wiki/Infrastructure/Factory2/Focus/Greenwave"
    __obj__ = "Gating Decisions"
    __icon__ = "https://apps.fedoraproject.org/img/icons/greenwave.png"

    @staticmethod
    def satisfied(msg):
        try:
            # https://pagure.io/greenwave/pull-request/100
            return msg['msg']['policies_satisfied']
        except KeyError:
            return msg['msg']['policies_satisified']

    def link(self, msg, **config):
        subject = msg['msg']['subject']
        if subject:
            base = "https://taskotron.fedoraproject.org/resultsdb/results"
            query = urlencode(msg['msg']['subject'][0])
            return base + "?" + query

    def subtitle(self, msg, **config):
        if self.satisfied(msg):
            decision = self._("is a GO")
        else:
            decision = self._("says NO-GO")

        tmpl = self._(
            "greenwave {decision} on {item} for "
            "\"{decision_context}\" ({product_version})"
        )
        subject = msg['msg']['subject']
        items = [entry.get('item') for entry in subject if entry.get('item')]
        item = items[0] if items else "\"something\""
        return tmpl.format(decision=decision, item=item, **msg['msg'])

    def secondary_icon(self, msg, **config):
        packages = self.packages(msg, **config)
        if packages:
            primary = sorted(packages)[0]
            base = 'https://apps.fedoraproject.org/packages/images/icons/'
            return base + primary + '.png'

    def packages(self, msg, **config):
        subject = msg['msg']['subject']
        items = [
            entry.get('item') for entry in subject
            if entry.get('item') and entry.get('type') == 'koji_build'
        ]
        return set([item.rsplit('-', 2)[0] for item in items])



# This file is part of fedmsg.
# Copyright (C) 2017 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for Greenwave messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestGreenwaveDecisionChange(Base):
    """ These messages are published by the `greenwave
    <http://fedoraproject.org/wiki/Infrastructure/Factory2/Focus/Greenwave>`_
    when some event causes it to change its decision about an artifacts.

    Here we have an example of a positive decision change on whether or not
    Bodhi should allow libmtp-1.1.14-1.fc27 to go to the stable repos.
    """
    expected_title = "greenwave.decision.update"
    expected_subti = "greenwave is a GO on libmtp-1.1.14-1.fc27 for " + \
        "\"bodhi_update_push_stable\" (fedora-27)"
    expected_link = "https://taskotron.fedoraproject.org/resultsdb/" + \
        "results?item=libmtp-1.1.14-1.fc27&type=koji_build"
    expected_icon = "https://apps.fedoraproject.org/img/icons/greenwave.png"
    expected_secondary_icon = ("https://apps.fedoraproject.org/packages/"
                               "images/icons/libmtp.png")
    expected_packages = set(['libmtp'])

    msg = {
        "i": 3,
        "username": "openshift",
        "timestamp": 1508872576.0,
        "msg_id": "2017-5ee197f3-dcc7-4cd6-af54-c5c448e0fe94",
        "crypto": "x509",
        "topic": "org.fedoraproject.prod.greenwave.decision.update",
        "msg": {
            "policies_satisified": True,
            "decision_context": "bodhi_update_push_stable",
            "product_version": "fedora-27",
            "applicable_policies": [
                "taskotron_release_critical_tasks_for_stable"
            ],
            "unsatisfied_requirements": [],
            "subject": [
                {
                    "item": "libmtp-1.1.14-1.fc27",
                    "type": "koji_build"
                }
            ],
            "summary": "all required tests passed",
            "previous": {
                "policies_satisified": False,
                "summary": "1 of 2 required tests not found",
                "unsatisfied_requirements": [
                    {
                        "testcase": "dist.upgradepath",
                        "item": {
                            "item": "libmtp-1.1.14-1.fc27",
                            "type": "koji_build"
                        },
                        "type": "test-result-missing"
                    }
                ],
                "applicable_policies": [
                    "taskotron_release_critical_tasks_for_stable"
                ]
            }
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class ByUser(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return 'ansible.playbook' in msg['topic']

    def possessive(self, name):
        """ Given a name, return the possessive form. """
        return self._("{name}'s").format(name=name)

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        agent = ms[0]['msg']['userid']
        playbooks = set([m['msg']['playbook'].split('/')[-1] for m in ms])
        playbooks_text = self.list_to_series(playbooks)

        N = len([m for m in ms if m['topic'].endswith('start')])
        if not N:
            N = len(ms)

        playbooks_predicate = 'playbooks' if len(playbooks) > 1 else 'playbook'
        times_predicate = 'times' if N > 1 else 'time'

        subtitle = '{agent} ran the {playbooks} ' + \
            '{playbooks_predicate} {N} {times_predicate}'

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            agent=agent, playbooks=playbooks_text, N=N,
            playbooks_predicate=playbooks_predicate,
            times_predicate=times_predicate)

        subjective_agent = 'You' if agent == subject else agent
        tmpl['subjective'] = subtitle.format(
            agent=subjective_agent, playbooks=playbooks_text, N=N,
            playbooks_predicate=playbooks_predicate,
            times_predicate=times_predicate)

        default = tmpl['icon']

        tmpl['secondary_icon'] = self.get_secondary_icon(constituents, default)
        tmpl['link'] = 'http://infrastructure.fedoraproject.org' + \
            '/infra/ansible.git/'

        return tmpl

    def matches(self, a, b, **config):
        """ The changes must all be **to** the same pkgdb user """
        a, b = a['msg'], b['msg']
        if a['userid'] != b['userid']:
            return False
        return True

    def get_secondary_icon(self, constituents, default):
        userid = constituents[0]['msg']['userid']
        return avatar_url(userid)

    def get_link(self, constituents):
        return ''



# This file is part of fedmsg.
# Copyright (C) 2016 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Adam Williamson <awilliam@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor

class OpenQAProcessor(BaseProcessor):
    __name__ = "openqa"
    __description__ = "Distribution-level automated testing"
    __link__ = "https://openqa.fedoraproject.org"
    __docs__ = "https://os-autoinst.github.io/openQA/documentation/"
    __obj__ = "openQA test results"
    __icon__ = "https://apps.fedoraproject.org/img/icons/openqa.png"

    def _msg_type(self, msg):
        """We often have to handle different types of messages in
        entirely different ways; this is the identification logic, so
        we don't keep repeating it.
        """
        if '.job.' in msg['topic']:
            return 'job'
        if '.comment.' in msg['topic']:
            return 'comment'

    def subtitle(self, msg, **config):
        if self._msg_type(msg) == 'job':
            return self._subtitle_job(msg, **config)
        elif self._msg_type(msg) == 'comment':
            return self._subtitle_comment(msg, **config)

    def _subtitle_job(self, msg, **config):
        job = msg['msg'].get('id', '')
        # old messages had 'build' not 'BUILD'
        build = msg['msg'].get('BUILD', msg['msg'].get('build', ''))
        remain = msg['msg'].get('remaining', '')
        result = msg['msg'].get('result')
        # all the following do not exist in old messages so we always
        # check before using them
        test = msg['msg'].get('TEST')
        arch = msg['msg'].get('ARCH')
        machine = msg['msg'].get('MACHINE')
        flavor = msg['msg'].get('FLAVOR')
        iso = msg['msg'].get('ISO')
        hdd1 = msg['msg'].get('HDD_1')

        msgtmpl = ''
        if ".stg" in msg['topic']:
            msgtmpl = "staging "

        msgtmpl += "job {0} ".format(job)
        if test:
            # if we have 'test' we can assume 'machine'
            msgtmpl += "(test {0} on {1}".format(test, machine)
            if iso:
                msgtmpl += " for iso {0}) ".format(iso)
            elif hdd1:
                msgtmpl += " for disk {0}) ".format(hdd1)
            else:
                # just for general safety...
                msgtmpl += ") "

        if msg['topic'].endswith('job.duplicate'):
            auto = msg['msg'].get('auto', '')
            if auto == "0":
                auto = "manually "
            if auto == "1":
                auto = "automatically "
            msgtmpl += "{0}duplicated as {1} for {2}"
            return msgtmpl.format(auto, result, build)

        if msg['topic'].endswith('job.restart'):
            msgtmpl += "restarted as {0} for {1}"
            return msgtmpl.format(result, build)

        if msg['topic'].endswith('job.done'):
            msgtmpl += "completed for {0}, {1} remaining jobs"
            return msgtmpl.format(build, remain)

    def _subtitle_comment(self, msg, **config):
        # get the info
        extract = msg['msg'].get('text', '')
        if len(extract) > 30:
            extract = extract[:30] + "..."
        user = msg['msg'].get('user', 'someone')
        comment = msg['msg'].get('id')
        job = msg['msg'].get('job_id')
        group = msg['msg'].get('group_id')
        action = 'did something to'
        if msg['topic'].endswith('comment.create'):
            action = 'created'
        elif msg['topic'].endswith('comment.update'):
            action = 'updated'
        elif msg['topic'].endswith('comment.delete'):
            action = 'deleted'

        # construct the message
        subtitle = "{0} {1} comment #{2} on openQA".format(user, action, comment)
        if job:
            subtitle += " job {0}".format(job)
        elif group:
            subtitle += " group {0}".format(group)
        if extract:
            subtitle += ", text: {0}".format(extract)

        # return it
        return subtitle

    def link(self, msg, **config):
        if '.stg' in msg['topic']:
            baseurl = 'https://openqa.stg.fedoraproject.org'
        else:
            baseurl = 'https://openqa.fedoraproject.org'

        if self._msg_type(msg) == 'job':
            # job messages
            urltmpl = '{0}/tests/{1}'
            return urltmpl.format(baseurl, msg['msg'].get('id'))

        elif self._msg_type(msg) == 'comment':
            # comment messages
            job = msg['msg'].get('job_id')
            group = msg['msg'].get('group_id')
            if job:
                urltmpl = '{0}/tests/{1}#comments'
                return urltmpl.format(baseurl, job)
            elif group:
                # comments are just right on the page
                urltmpl = '{0}/group_overview/{1}'
                return urltmpl.format(baseurl, group)
            else:
                # we have no idea, just return openQA
                return baseurl

    def objects(self, msg, **config):
        objs = []
        if 'build' in msg['msg']:
            objs.append(msg['msg']['build'])
        elif 'BUILD' in msg['msg']:
            objs.append(msg['msg']['BUILD'])

        if 'ISO' in msg['msg']:
            objs.append(msg['msg']['ISO'])
        # HDD_1 is probably not an 'object' if there's an ISO, it's
        # more likely just a test component
        elif 'HDD_1' in msg['msg']:
            objs.append(msg['msg']['HDD_1'])

        if msg['msg'].get('job_id'):
            objs.append(msg['msg']['job_id'])
        if msg['msg'].get('group_id'):
            objs.append(msg['msg']['group_id'])

        return set(objs)

    def usernames(self, msg, **config):
        # comment messages have a 'user', job messages don't
        user = msg['msg'].get('user')
        if user:
            return set((user,))
        else:
            return set()
